<?php

/**
 * @file googlebase_json.inc
 *
 */

abstract class googlebase_json {
  public $data = NULL;
  protected $namespaces = array(
    'g' => 'http://base.google.com/ns/1.0',
  );

  protected $base_element = 'entry';

  function __construct(&$data = array(), $namespaces = NULL) {
    if (!empty($namespaces)) {
      $this->namespaces = array_merge($this->namespaces, $namespaces);
    }

    if (empty($data)) {
      $data['version'] = '1.0';
      $data['encoding'] = 'UTF-8';
      $data['feed'] = array();
      $data['feed']['xmlns'] = 'http://www.w3.org/2005/Atom';

      foreach ($this->namespaces as $prefix => $namespace) {
        $data['feed']['xmlns$' . $prefix] = $namespace;
      }
      $this->data = $data;
    }
  }

  public function add_element($name, $value = NULL, $attributes = array()) {
    $this->data[$name]['$t'] = $value;
    foreach ($attributes as $attribute => $value) {
      $this->data[$name][$attribute]= $value;
    }
  }

  public function add_namespaced_element($namespace, $name, $value = NULL, $attributes = array()) {
    if (empty($this->namespaces[$namespace])) {
      return FALSE;
    }
    $this->data[$namespace . '$' . $name]['$t'] = $value;
    foreach ($attributes as $attribute => $value) {
      $this->data[$namespace . '$' . $name][$attribute]= $value;
    }
  }

  /*
   * Adds a Google Base attribute (g: namespace);
   */
  public function add_attribute($name, $value, $type = NULL) {
    $this->data['g$' . $name]['$t'] = $value;
    if (!empty($type)) {
      $this->data['g$' . $name]['type']= $value;
    }
  }

  public function __toString() {
    return json_encode($this->data);
  }

  public function get_xml() {
    return json_encode($this->data);
  }

  public function get_json() {
    return json_encode($this->data);
  }
}