<?php

/**
 * Field handler to provide simple renderer that allows linking to a node.
 */
class googlebase_views_handler_filter_location_distance extends views_handler_filter {
  var $no_single = TRUE;
  function option_definition() {
    $options = parent::option_definition();

    $options['operator']['default'] = 'optional';

    return $options;
  }

  /**
   * Provide simple equality operator
   */
  function operator_form(&$form, &$form_state) {
    parent::operator_form($form, $form_state);
    $form['operator'] = array(
      '#type' => 'radios',
      '#title' => t('On empty input'),
      '#default_value' => $this->operator,
      '#options' => array(
        'optional' => t('Show All'),
        'required' => t('Show None'),
      ),
    );
  }

  /**
   * Provide simple equality operator
   */
  function options_form(&$form, &$form_state) {
    parent::options_form($form, $form_state);
    $form['units'] = array(
      '#type' => 'radios',
      '#title' => t('Distance units'),
      '#default_value' => $this->options['units'],
      '#options' => array(
        'mi' => t('Miles'),
        'km' => t('Kilometers'),
        'm' => t('Meters'),
      ),
    );
  }

  /**
   * Add a type selector to the value form
   */
  function value_form(&$form, &$form_state) {
    parent::value_form($form, $form_state);
    if (empty($form_state['exposed'])) {
      $form['value'] = array(
        '#title' => t('Distance'),
        '#type' => 'textfield',
        '#size' => 3,
        '#default_value' => $this->value['distance'],
        '#attributes' => array('title' => t('Enter a distance as an integer.')),
        '#description' => t('Enter a distance as an integer.'),
        '#field_suffix' => $this->options['units'],
      );
    }
    
  }

  /**
   * Provide a location text field and optional distance field
   */
  function exposed_form(&$form, &$form_state) {
    parent::exposed_form($form, $form_state);
    if (isset($this->options['expose']['identifier'])) {
      $key = $this->options['expose']['identifier'];

      if ($this->options['units'] == 'mi') {
        $suffix = 'miles';
      }
      elseif ($this->options['units'] == 'km') {
        $suffix = 'kilometers';
      }
      elseif ($this->options['units'] == 'm') {
        $suffix = 'meters';
      }
      if ($this->options['expose']['distance'] == 'select') {
        $form[$key] = array(
          '#type' => 'select',
          '#default_value' => $this->value,
          '#options' => array(
            '5' => "5 $suffix",
            '10' => "10 $suffix",
            '25' => "25 $suffix",
            '50' => "50 $suffix",
            '100' => "100 $suffix",
          ),
        );
      }
      elseif ($this->options['expose']['distance'] == 'text') {
        $form[$key] = array(
          '#type' => 'textfield',
          '#size' => 3,
          '#default_value' => $this->value,
          '#attributes' => array('title' => t('Enter a distance as an integer.')),
          '#field_suffix' => $suffix,
        );
      }
    }
  }

  function expose_options() {
    parent::expose_options();
    $this->options['expose']['distance'] = FALSE;
  }

  function expose_form_right(&$form, &$form_state) {
    parent::expose_form_right($form, $form_state);
    $form['expose']['distance'] = array(
      '#type' => 'radios',
      '#title' => t('Exposed distance field'),
      //'#description' => t('If checked, the only items presented to the user will be the ones selected here.'),
      '#default_value' => $this->options['expose']['distance'],
      '#options' => array(
        'select' => t('Select box'),
        'text' => t('Text field'),
      ),
    );
  }

  /**
   * Validate the options form.
   */
  function exposed_validate($form, &$form_state) {

  }

  /**
   * Add this filter to the query.
   *
   * Due to the nature of fapi, the value and the operator have an unintended
   * level of indirection. You will find them in $this->operator
   * and $this->value respectively.
   */
  function query() {
    if (!empty($this->value)) {
      // [location: @"San Francisco, CA, USA" + 3mi]
      if ($location = $this->query->get_structure('location')) {
        $location = str_replace(']', ' + ' . $this->value . $this->options['units'] . ']', $location);
        $this->query->add_structure('location', $location);
      }
    }
  }
}
